<?php

namespace Intraservis\Application;

use AccessDeniedController;
use AccessDeniedException;
use ErrorController;
use Intraservis\DB\Exceptions\NotFoundException;
use Intraservis\DI\Container;
use Intraservis\HTTP\Request;
use Intraservis\HTTP\Response;
use Intraservis\Security\Exceptions\BadLoginPasswordException;
use Intraservis\Security\Exceptions\UserNotFoundException;
use Intraservis\Security\User;

class Application
{
	/**
	 * @var Container
	 */
	private Container $container;

	/**
	 * @var Request
	 */
	private Request $request;

	/**
	 * @var Response
	 */
	private Response $response;

	/**
	 * @var User
	 */
	private User $user;

	private $configuration;


	/**
	 * Konštruktor.
	 *
	 * @param array associatívne pole s nastaveniami
	 */
	public function __construct(Container $container, Request $request, Response $response, $configuration)
	{
		$this->container = $container;
		$this->request = $request;
		$this->response = $response;
		$this->configuration = $configuration;

		if (session_status() !== PHP_SESSION_ACTIVE) {
			ini_set('session.use_cookies', '1');
			ini_set('session.use_only_cookies', '1');
			ini_set('session.use_trans_sid', '0');
			ini_set('session.cookie_path', '/');
			ini_set('session.cookie_httponly', '1');
			session_start();
		}

		$this->user = $this->container->getService('user');
		$this->user->autologin();

		if ($this->user->isLoggedIn === true) {
			$this->user->identity->userModel->user = $this->user;
		} else {
			if (isset($_SESSION['user']) == true) {
				unset($_SESSION['user']);
			}
		}
	}


	public function run()
	{
		if (
			$this->user->isLoggedIn === false
			and empty($_SERVER['PHP_AUTH_USER']) === false
			and empty($_SERVER['PHP_AUTH_PW']) === false
		) {
			try {
				$this->user->login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']);
			} catch (BadLoginPasswordException $e) {
			} catch (UserNotFoundException $e) {
			}
		}

		// TODO: presunúť do Controllera, tu to nemá byť
		if ($this->checkBrand($this->configuration['brandName']) === false) {
			$this->configuration['brandName'] = 'default';
			$controllerClass = 'InvalidBrandController';
		} else {
			$controllerClass = $this->configuration['moduleName'] . 'Controller';
		}

		if (class_exists($controllerClass) === false) {
			$controller = new \ErrorController("Neznámy controller $controllerClass", null, 404);
		} else {
			/**
			 * @var DynamicController
			 */
			$controller = new $controllerClass;
		}

		$controller->configuration = $this->configuration;
		$this->container->injectDepencies($controller);

		try {
			$controller->show();
		} catch (NotFoundException $e) {
			if ($this->configuration["development"]) {
				bdump($e->where);
				bdump($e->sql);
				throw $e;
			}

			$message = 'Položka s ktorou sa pokúšate pracovať neexistuje alebo nemáte oprávnenie.';

			if (PHP_SAPI === 'cli') {
				echo "\n" . $message . "\n";
				exit(1);
			}

			$controller = new \ErrorController($message, null, 404);
			$controller->configuration = $this->configuration;
			$this->container->injectDepencies($controller);
			$controller->show();
		} catch (AccessDeniedException $e) {
			if (PHP_SAPI === 'cli') {
				$message = $e->getMessage();
				if (strlen($message) === 0) {
					throw $e;
				}

				echo "\n" . $message . "\n";
				exit(1);
			}

			if ($this->configuration["development"]) {
				throw $e;
			}

			$controller = new AccessDeniedController($e);
			$controller->configuration = $this->configuration;
			$this->container->injectDepencies($controller);
			$controller->show();
			exit;
		}
	}


	private function checkBrand($brandName)
	{
		if ($brandName === false) {
			return false;
		}

		if (preg_match('/^([a-z_-]+)$/', $brandName) === false) {
			return false;
		} else if ($brandName == 'default') {
			return true;
		}

		$personModel = $this->container->getService('PersonModel');
		try {
			$personModel->where([['brand', $brandName]])->fetch();
			return true;
		} catch (NotFoundException $e) {
			return false;
		}
	}
}
