<?php

namespace Intraservis\Application;

use Intraservis\HTTP\Request;
use Intraservis\Utils\Arrays;
use Intraservis\Utils\ImmutableArray;
use Nette\Neon\Neon;

class Configurator
{
	private Request $request;

	private array $sysConfiguration;


	public function __construct(Request $request, array $sysConfiguration)
	{
		$this->request = $request;
		$this->sysConfiguration = $sysConfiguration;
	}


	public function getConfiguration()
	{
		if (PHP_SAPI === 'cli') {
			return $this->getCliConfiguration();
		}

		return $this->getHttpConfiguration();
	}


	public function getHttpConfiguration()
	{
		$host = $this->request->getHost();

		// Vyparsovanie názvu portálu a brandu
		if (array_key_exists($host, $this->sysConfiguration['domainMappings']) === true) {
			list($siteName, $brandName) = $this->sysConfiguration['domainMappings'][$host];
		} else {
			$baseDomain = $this->sysConfiguration['baseDomain'];
			$baseDomain = str_replace('.', '\\.', $baseDomain);

			$pattern = '/^(www\\.)?((?P<brand>[\\w\\-]+)\\.)?((?P<site>[\\w\\-]+)\\.)' . $baseDomain . '$/';

			$siteName = false;
			$brandName = false;
			if (preg_match($pattern, $host, $matches)) {
				if (empty($matches['brand']) === false) {
					$brandName = $matches['brand'];
				} else {
					$brandName = 'default';
				}

				$brandName = $brandName === 'www' ? 'default' : $brandName;
				$siteName = $matches['site'];
			}
		}

		// Načítanie konfigurácie portálu
		if (isset($this->sysConfiguration['namespaces'][$siteName]) === false) {
			http_response_code(404);
			die('Unknown site');
		}

		$siteName = $this->sysConfiguration['namespaces'][$siteName];

		$configDir = $this->sysConfiguration['dirs']['config'] . "/Sites/$siteName/Configuration";

		// aplikačná konfigurácia
		$configurationFile = $configDir . '/config.neon';
		$configuration = Neon::decode(file_get_contents($configurationFile));

		// lokálna konfigurácia
		$localConfigurationFile = $configDir . '/config.local.neon';
		$localConfiguration = Neon::decode(file_get_contents($localConfigurationFile));

		// Doplnenie lokálnej konfigurácie do konfigurácie portálu
		$configuration = array_merge_recursive($configuration, $localConfiguration);

		// Doplnenie globálnej konfigurácie do konfigurácie portálu
		$configuration = array_merge_recursive($this->sysConfiguration, $configuration);

		// doplnenie ciest k adresárom na absolútne cesty (ak je treba)
		foreach ($configuration['dirs'] as $key => $dir) {
			if ($dir[0] === '/' or $key === 'base' or $key === 'config') continue;
			$configuration['dirs'][$key] = $configuration['dirs']['base'] . '/' . rtrim($dir, '/');
		}

		// Doplnenie názvu portálu, brandu, menného priestoru a aktuálneho modulu do konfigurácie
		$configuration['siteName'] = $siteName;
		$configuration['brandName'] = $brandName;
		$configuration['namespace'] = $siteName;

		$resource = $this->request->getResourceArray();

		$moduleName = Arrays::getValue($resource, 0);
		if (empty($moduleName) === true) {
			$moduleName = 'Welcome';
		} else {
			array_shift($resource);
		}

		if (isset($configuration['modules']['aliases'][$moduleName]) === true) {
			$moduleName = $configuration['modules']['aliases'][$moduleName];
		}

		$configuration['moduleName'] = $moduleName;

		$moduleAction = Arrays::getValue($resource, 0);
		if (empty($moduleAction) === true) {
			$moduleAction = 'index';
		} else {
			array_shift($resource);
		}
		$configuration['moduleAction'] = $moduleAction;
		$configuration['moduleActionParams'] = $resource;

		// spracovanie konfigurácie konfigurátorom portálu ak je nadefinovaný
		if (empty($configuration['configurator']) === false) {
			$siteConfigurator = new $configuration['configurator'];
			$configuration = $siteConfigurator->getConfiguration($configuration);
		}

		// autoloader
		// doplnenie relatívnej cesty na absolútnu cestu
		foreach ($configuration['autoloader']['dirs'] as &$dir) {
			$dir = $configuration['dirs']['base'] . '/' . $dir;
		}
		// doplnenie ciest povolených modulov do konfigurácie autoloadera
		foreach ($configuration['modules']['load'] as $moduleName) {
			if (is_array($moduleName) === true) {
				$moduleName = $moduleName['name'];
			}

			$configuration['autoloader']['dirs'][] = $configuration['dirs']['base'] . '/app/Modules/' . $moduleName;
		}
		// registrácia
		$loader = new \Nette\Loaders\RobotLoader;
		$loader->setTempDirectory($configuration['dirs']['cache']);
		foreach ($configuration['autoloader']['dirs'] as $dir) {
			$loader->addDirectory($dir);
		}
		$loader->register();
		// registrácia špeciálneho autoloadera
		spl_autoload_register(function($class) use ($configuration) {
			if (array_key_exists($class, $configuration["autoloader"]["files"]) === true) {
				require_once($configuration['dirs']['base'] . "/" . $configuration["autoloader"]["files"][$class]);
			}
		});

		$configuration = Arrays::setValues($configuration, $configuration);
		$configuration = new \Intraservis\Utils\ImmutableArray($configuration);
		$this->configuration = $configuration;

		return $configuration;
	}


	public function getCliConfiguration()
	{
		$shortopts = '';
		$shortopts .= 's::';
		$shortopts .= 'b::';
		$shortopts .= 'm:';
		$shortopts .= 'a:';

		$longopts  = [
			'site::',
			'brand::',
			'module:',
			'action:',
		];

		$arguments = getopt($shortopts, $longopts);

		$siteName = 'mojoperator';
		$brandName = 'default';

		if (empty($arguments['s']) === false) {
			$siteName = $arguments['s'];
		}

		if (empty($arguments['site']) === false) {
			$siteName = $arguments['site'];
		}

		if (empty($arguments['b']) === false) {
			$brandName = $arguments['b'];
		}

		if (empty($arguments['brand']) === false) {
			$brandName = $arguments['brand'];
		}

		// Načítanie konfigurácie portálu
		if (isset($this->sysConfiguration['namespaces'][$siteName]) === false) {
			die('Unknown site');
		}

		$configDir = $this->sysConfiguration['dirs']['config'] . '/Sites/' . $this->sysConfiguration['namespaces'][$siteName] . '/Configuration';

		// aplikačná konfigurácia
		$configurationFile = $configDir . '/config.neon';
		$configuration = Neon::decode(file_get_contents($configurationFile));

		// lokálna konfigurácia
		$localConfigurationFile = $configDir . '/config.local.neon';
		$localConfiguration = Neon::decode(file_get_contents($localConfigurationFile));

		// Doplnenie lokálnej konfigurácie do konfigurácie portálu
		$configuration = array_merge_recursive($configuration, $localConfiguration);

		// Doplnenie globálnej konfigurácie do konfigurácie portálu
		$configuration = array_merge_recursive($this->sysConfiguration, $configuration);

		// doplnenie ciest k adresárom na absolútne cesty (ak je treba)
		if ($configuration['dirs']['data'][0] !== '/') {
			$configuration['dirs']['data'] = $configuration['dirs']['base'] . '/' . $configuration['dirs']['data'];
		}

		if ($configuration['dirs']['cache'][0] !== '/') {
			$configuration['dirs']['cache'] = $configuration['dirs']['base'] . '/' . $configuration['dirs']['cache'];
		}

		if ($configuration['dirs']['public_cache'][0] !== '/') {
			$configuration['dirs']['public_cache'] = $configuration['dirs']['base'] . '/' . $configuration['dirs']['public_cache'];
		}
		if ($configuration['dirs']['log'][0] !== '/') {
			$configuration['dirs']['log'] = $configuration['dirs']['base'] . '/' . $configuration['dirs']['log'];
		}

		// Doplnenie názvu portálu, brandu, menného priestoru a aktuálneho modulu do konfigurácie
		$configuration['siteName'] = $siteName;
		$configuration['brandName'] = $brandName;
		$configuration['namespace'] = $this->sysConfiguration['namespaces'][$siteName];

		if (empty($arguments['m']) === false) {
			$moduleName = $arguments['m'];
		}

		if (empty($arguments['module']) === false) {
			$moduleName = $arguments['module'];
		}

		if (empty($moduleName) === true) {
			$moduleName = 'Welcome';
		}

		if (isset($configuration['modules']['aliases'][$moduleName]) === true) {
			if (is_array($moduleName) === true) {
				$moduleName = $moduleName['name'];
			}

			$moduleName = $configuration['modules']['aliases'][$moduleName];
		}

		$configuration['moduleName'] = $moduleName;

		if (empty($arguments['a']) === false) {
			$moduleAction = $arguments['a'];
		}

		if (empty($arguments['action']) === false) {
			$moduleAction = $arguments['action'];
		}

		if (empty($moduleAction) === true) {
			$moduleAction = 'index';
		}

		$configuration['moduleAction'] = $moduleAction;
		$configuration['moduleActionParams'] = $arguments;

		// spracovanie konfigurácie konfigurátorom portálu ak je nadefinovaný
		if (empty($configuration['configurator']) === false) {
			$siteConfigurator = new $configuration['configurator'];
			$configuration = $siteConfigurator->getConfiguration($configuration);
		}

		// autoloader
		// doplnenie relatívnej cesty na absolútnu cestu
		foreach ($configuration['autoloader']['dirs'] as &$dir) {
			$dir = $configuration['dirs']['base'] . '/' . $dir;
		}
		// doplnenie ciest povolených modulov do konfigurácie autoloadera
		foreach ($configuration['modules']['load'] as $moduleName) {
			if (is_array($moduleName) === true) {
				$moduleName = $moduleName['name'];
			}

			$configuration['autoloader']['dirs'][] = $configuration['dirs']['base'] . '/app/Modules/' . $moduleName;
		}
		// registrácia
		$loader = new \Nette\Loaders\RobotLoader;
		$loader->setTempDirectory($configuration['dirs']['cache']);
		foreach ($configuration['autoloader']['dirs'] as $dir) {
			$loader->addDirectory($dir);
		}
		$loader->register();
		// registrácia špeciálneho autoloadera
		spl_autoload_register(function($class) use ($configuration) {
			if (array_key_exists($class, $configuration["autoloader"]["files"]) === true) {
				require_once($configuration['dirs']['base'] . "/" . $configuration["autoloader"]["files"][$class]);
			}
		});

		$configuration = Arrays::setValues($configuration, $configuration);
		$configuration = new ImmutableArray($configuration);

		return $configuration;
	}
}
